#region File Description
//-----------------------------------------------------------------------------
// Font.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#define MOUSE  //uncomment to enable mouse - see mouse tutorial

#region Using Statements
using System;
using System.IO;
using System.Xml.Serialization;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Input;
using System.Text;
using System.Collections.Generic;
using Microsoft.Xna.Framework.GamerServices;
#endregion

namespace Snowflakes
{
    /// <summary>
    /// The current animation state of the board and of individual snowflakes
    /// </summary>
    public enum Animation
    {
        /// <summary>
        /// No animations happening
        /// </summary>
        None,
        /// <summary>
        /// Some snowflakes are currently breaking
        /// </summary>
        Breaking,
        /// <summary>
        /// Need a new time, time is up
        /// </summary>
        TimeUp,
        /// <summary>
        /// Breaking snowflakes are all gone
        /// </summary>
        Gone,
    }

    /// <summary>
    /// The current animation state of the board and of individual snowflakes
    /// </summary>
    public enum InfoScreen
    {
        /// <summary>
        /// No info screen required
        /// </summary>
        None,
        /// <summary>
        /// Change from matching shapes to matching colors or vice versa
        /// </summary>
        SwitchMatch,
        /// <summary>
        /// First time they matched a wrong type together
        /// </summary>
        WrongMatch,
        /// <summary>
        /// Show initial instuctions
        /// </summary>
        Instructions,
        /// <summary>
        /// Information for pause menu
        /// </summary>
        PauseInfo,
        /// <summary>
        /// Error information when get first time up
        /// </summary>
        TimeUp,
    }

    /// <summary>
    /// The game board holds the pieces, the cursor and handles most of the game logic
    /// </summary>
    public class GameBoard : DrawableGameComponent
    {
        /// <summary>
        /// Number if snowflakes in the x direction
        /// </summary>
        public const int Width = 8;
        /// <summary>
        /// Number of snowflakes in the y direction
        /// </summary>
        public const int Height = 1;

        private const int LeftEdge = 50;
        private const int TopEdge = 288;
        private const int scoreAmount = 10;
        private const int bonusExactMatch = 2;
        private const int bonusClearScreen = 50;

        /// <summary>
        /// Snowflakes stores the set of snowflakes used on the game board
        /// </summary>
        internal Snowflake[,] Snowflakes = new Snowflake[Width, Height];
        private static Vector2 SnowflakeToMatchPosition = new Vector2(312, 36);
        internal Snowflake SnowflakeToMatch;
        Random random = new Random();
        private bool instructionPhase;
        private bool hadFirstWrongShape;
        private bool hadFirstWrongColor;
        private bool hadFirstWrongConvert;
        private bool hadFirstTimeUpWrong;
        private bool hadFirstSwitchInMiddleOfScreen;
        private bool hadFirstSwitchEveryTime;

        /// <summary>
        /// Animation state stores what the game board is currently doing at any moment 
        /// in time
        /// </summary>
        internal Animation AnimationState = Animation.None;
        public InfoScreen InfoScreenState = InfoScreen.None;
        private bool gameOver;

        /// <summary>
        /// Cursor position and texture
        /// </summary>
        private int cursorX;
        private int cursorY;
        private Texture2D snowflakeCursorTexture;

        /// <summary>
        /// Score information
        /// </summary>
        private Vector2 scoreOffset = new Vector2(0, -1.0f * Snowflake.Height);
        private GameTime scoreFadeStart;
        private float scoreFadeFactor;
        private float scoreFadeDistance = Snowflake.Height * 2;
        private int totalSelected;

        /// <summary>
        /// Match timer information
        /// </summary>
        public double timeAllowedToMatch;
        public double maxTimeAllowedToMatch = 30;
        public double minTimeAllowedToMatch = 3;
        public double timeAllowedToMatchDecrease = 1;
        private const int breakFrameCount = 12;
        private double matchStartTime;
        private double timeLeftAtPause;
        private int timeLeftInteger;
        private int changeTimeAmount;
        private int changeTimeOccurence = 3;
        public Texture2D timeSegmentImg;
        public Texture2D timeSegmentImgSmall;
        private int pauseTime = 1;// Number of seconds to pause after timeout and illegal move

        /// <summary>
        /// Sounds
        /// </summary>
        SoundEffect soundEffectMatch;
        SoundEffect soundClearBonus;
        SoundEffect soundClear;
        SoundEffect soundClearIllegal;
        SoundEffect soundNewScreen;


#if MOUSE
        private int lastMouseX;
        private int lastMouseY;
        private bool leftButtonDown;
        private bool rightButtonDown;
#endif

        /// <summary>
        /// Create a new game board
        /// </summary>
        /// <param name="game"></param>
        public GameBoard(Game game)
            : base(game)
        {
            Snowflake.Initialize();
            changeTimeAmount = changeTimeOccurence;
#if MOUSE
            MouseState mouse = Mouse.GetState();
            lastMouseX = mouse.X;
            lastMouseY = mouse.Y;
#endif
        }



        /// <summary>
        /// Load graphics content.
        /// </summary>
        protected override void LoadContent()
        {
            snowflakeCursorTexture =
                SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake_cursor");
            Snowflake.LoadContent();

            soundEffectMatch = SnowflakesGame.Content.Load<SoundEffect>("Audio/switch_match_type");
            soundClearBonus = SnowflakesGame.Content.Load<SoundEffect>("Audio/clear_bonus2");
            soundClear = SnowflakesGame.Content.Load<SoundEffect>("Audio/clear_normal");
            soundClearIllegal = SnowflakesGame.Content.Load<SoundEffect>("Audio/clear_illegal2");
            soundNewScreen = SnowflakesGame.Content.Load<SoundEffect>("Audio/new_screen");
            timeSegmentImg = SnowflakesGame.Content.Load<Texture2D>("Textures/time_segment_switch_match");
            timeSegmentImgSmall = SnowflakesGame.Content.Load<Texture2D>("Textures/time_segment_small");

            base.LoadContent();
        }

        /// <summary>
        /// Checks to see if the game is over yet
        /// </summary>
        public bool GameOver
        {
            get
            {
                if (!gameOver && IsGameOver())
                {
                    gameOver = true;
                }

                return gameOver;
            }
        }

        /// <summary>
        /// Sets it up so instructions will be shown on initial errors.
        /// </summary>
        public void setShowInstructions()
        {
            InfoScreenState = InfoScreen.Instructions;
            instructionPhase = true;
            hadFirstWrongColor = false;
            hadFirstWrongShape = false;
            hadFirstTimeUpWrong = false;
            hadFirstWrongConvert = false;
            hadFirstSwitchInMiddleOfScreen = false;
            hadFirstSwitchEveryTime = false;
        }

        /// <summary>
        /// Checks to see whether an info screen needs to be shown
        /// </summary>
        public bool needToShowInfo()
        {
            if (InfoScreenState != InfoScreen.None)
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// Called when the GameComponent needs to be updated.  
        /// Game Board update performs animation on the snowflakes, checks for cursor 
        /// movement. Most of the game logic is here or called from here
        /// </summary>
        /// <param name="gameTime">Current game time</param>
        public override void Update(GameTime gameTime)
        {
            if (gameTime == null)
                return;

            base.Update(gameTime);

            if (!GameOver)
            {
                Snowflake.UpdateStatic(gameTime);

                foreach (Snowflake snowflake in Snowflakes)
                {
                    if (snowflake != null)
                    {
                        snowflake.Update(gameTime);
                    }
                }
                SnowflakeToMatch.Update(gameTime);

                switch (AnimationState)
                {
                    case Animation.Breaking:
                        {
                            //Fade the score
                            if (totalSelected > 0)
                            {
                                scoreFadeFactor = (float)((gameTime.TotalGameTime -
                                    scoreFadeStart.TotalGameTime).TotalSeconds /
                                    Snowflake.BreakTime);
                            }

                            //Wait until all snowflakes are broken.
                            bool stillBreaking = false;
                            foreach (Snowflake snowflake in Snowflakes)
                            {
                                if (snowflake != null)
                                {
                                    if (snowflake.Animation == Animation.Breaking)
                                    {
                                        stillBreaking = true;
                                        break;
                                    }
                                }
                            }
                            if (SnowflakeToMatch.Animation == Animation.Breaking)
                            {   // Still breaking the match snowflake, so need to keep waiting
                                stillBreaking = true;
                            }

                            //Done breaking 
                            if (!stillBreaking)
                            {
                                if (totalSelected > 0)
                                {   // Matched with another snowflake
                                    totalSelected = 0;
                                    SnowflakesGame.NumMatchesUntilSwitchType--;
                                    ClearBrokenSnowflakes();
                                    // Now check to see if need to switch match type
                                    if (SnowflakesGame.NumMatchesUntilSwitchType == 0)
                                    {// Should switch
                                        // Will need to pop up a screen to tell people
                                        if (instructionPhase == true)
                                        {
                                            // This is the first time we're switching
                                            // Need extra instructions
                                            InfoScreenState = InfoScreen.Instructions;
                                            // Instruction phase is over now
                                            instructionPhase = false;
                                            soundEffectMatch.Play();
                                        }
                                        else if (SnowflakesGame.SwitchFrequencyMatchType > 1)
                                        {   // Haven't reached end stage, so want to
                                            //  send out message.
                                            InfoScreenState = InfoScreen.SwitchMatch;
                                            soundEffectMatch.Play();
                                        }
                                        // else - reached end stage, no infoscreen needed.
                                        //  unless not shown the switch everytime message
                                        else if (!hadFirstSwitchEveryTime)
                                        {
                                            hadFirstSwitchEveryTime = true;
                                            InfoScreenState = InfoScreen.SwitchMatch;
                                        }
                                        else
                                        {
                                            // No info screen, but want to play the sound
                                            soundEffectMatch.Play();
                                        }
                                        if (SnowflakesGame.IsShapeMatch)
                                        {
                                            SnowflakesGame.IsShapeMatch = false;
                                        }
                                        else
                                        {
                                            SnowflakesGame.IsShapeMatch = true;
                                        }
                                        // Reset count
                                        SnowflakesGame.NumMatchesUntilSwitchType
                                                = SnowflakesGame.SwitchFrequencyMatchType;
                                    }
                                }
                                else
                                {
                                    // Else We were generating a new match snowflake
                                    // Don't need to check to change match type
                                }
                                AnimationState = Animation.None;
                                // Have cleared the match snowflake, so need a new
                                //  snowflake to match
                                SnowflakeToMatch = new Snowflake(random.Next());
                                SnowflakeToMatch.Position = SnowflakeToMatchPosition;
                                // Reset the time left
                                matchStartTime = gameTime.TotalGameTime.TotalSeconds;
                                timeLeftInteger = Convert.ToInt32(timeAllowedToMatch);
                            }
                            break;
                        }

                    case Animation.TimeUp:
                        {
                            // Check to see if could have made a match
                            if (aMatchExists())
                            {   // Decrease lives if could have
                                soundClearIllegal.Play();
                                //Sound.Play(SoundEntry.ClearIllegal);
                                SnowflakesGame.Lives--;
                                if (!hadFirstTimeUpWrong)
                                {
                                    hadFirstTimeUpWrong = true;
                                    InfoScreenState = InfoScreen.TimeUp;
                                }
                                // Pause game for one second
                                DateTime gameendtime;
                                gameendtime = DateTime.Now + TimeSpan.FromSeconds(pauseTime);
                                while (gameendtime < DateTime.Now)
                                {
                                    // Do nothing
                                }

                            }
                            // We need to generate a new match tile.
                            //  This means we need to break the snowflake
                            AnimationState = Animation.Breaking;
                            SnowflakeToMatch.Break(gameTime);
                            break;
                        }

                    case Animation.None:
                        {
                            if (InfoScreenState == InfoScreen.None)
                            {
                                if (InputHelper.GamePads[PlayerIndex.One].StartPressed)
                                {   // Player wants to pause
                                    InfoScreenState = InfoScreen.PauseInfo;
                                    // Remember the time we paused
                                    timeLeftAtPause = timeAllowedToMatch -
                                            (gameTime.TotalGameTime.TotalSeconds - matchStartTime);
                                }
                                else
                                {
                                    // Check whether need to make a new screen
                                    checkScreenClear(gameTime);
                                    UpdateMatchClock(gameTime);
                                    moveCursor();
                                    BreakSnowflakes(gameTime);
                                }
                            }
                            else
                            {  // Have an info screen up
                                // Find out what button is pressed, if any
#if MOUSE
                                MouseState mouse = Mouse.GetState();
                                bool leftButtonClicked = false;
                                bool rightButtonClicked = false;
                                if (mouse.LeftButton == ButtonState.Pressed)
                                {
                                    leftButtonDown = true;
                                }
                                if (mouse.RightButton == ButtonState.Pressed)
                                {
                                    rightButtonDown = true;
                                }
                                if (leftButtonDown && mouse.LeftButton == ButtonState.Released)
                                {
                                    leftButtonDown = false;
                                    leftButtonClicked = true;
                                }
                                if (rightButtonDown && mouse.RightButton == ButtonState.Released)
                                {
                                    rightButtonDown = false;
                                    rightButtonClicked = true;
                                }
#endif
                                bool APressed = InputHelper.GamePads[PlayerIndex.One].APressed
#if MOUSE
 || ((Game.IsMouseVisible && leftButtonClicked))
#endif
;
                                bool XPressed = InputHelper.GamePads[PlayerIndex.One].XPressed
#if MOUSE
 || (Game.IsMouseVisible && rightButtonClicked)
#endif
;
                                if (APressed)
                                {   // Have an info screen up
                                    // Wait until button A pressed OR left mouse button pushed down
                                    // Once button pressed
                                    if (InfoScreenState == InfoScreen.SwitchMatch
                                          || InfoScreenState == InfoScreen.TimeUp
                                          || InfoScreenState == InfoScreen.Instructions)
                                    {
                                        // Restart clock timer
                                        matchStartTime = gameTime.TotalGameTime.TotalSeconds;
                                        timeLeftInteger = Convert.ToInt32(timeAllowedToMatch);
                                    }
                                    else
                                    {   // Had a wrong match or paused the game
                                        // Recalculate start time, based on time that was left
                                        //  BEFORE the pause
                                        matchStartTime = (gameTime.TotalGameTime.TotalSeconds + timeLeftAtPause)
                                                            - timeAllowedToMatch;
                                    }
                                    InfoScreenState = InfoScreen.None;
                                }
                                else if (XPressed && InfoScreenState == InfoScreen.PauseInfo)
                                {
                                    // Need to quit the game, only if was on a pause
                                    InfoScreenState = InfoScreen.None;
                                    gameOver = true;
                                }
                                else if (InputHelper.GamePads[PlayerIndex.One].StartPressed)
                                {
                                    // Player has an info screen up, and wants to pause
                                    // Need to set time left at pause
                                    // WrongMatch state & PauseInfo already set the value,
                                    // All other states should have full amount of time left.
                                    if (InfoScreenState != InfoScreen.WrongMatch
                                            && InfoScreenState != InfoScreen.PauseInfo)
                                    {
                                        timeLeftAtPause = timeAllowedToMatch;
                                    }
                                    InfoScreenState = InfoScreen.PauseInfo;
                                }
                                // Else... no button pressed, do nothing
                            }
                            break;
                        }
                }
            }
        }

        /// <summary>
        /// Updates the clock for time left to make match
        /// </summary>
        /// <param name="time">Total game time</param>
        public void UpdateMatchClock(GameTime time)
        {
            double timeLeft = timeAllowedToMatch -
                                   (time.TotalGameTime.TotalSeconds - matchStartTime);
            timeLeftInteger = Convert.ToInt32(timeLeft);

            if (timeLeft < 0)
            {       // Time is up!
                AnimationState = Animation.TimeUp;
            }

        }


        /// <summary>
        /// Converts a board grid position into a pixel screen position 
        /// in 700x400 space
        /// </summary>
        /// <param name="x">0-number of snowflakes wide</param>
        /// <param name="y">0-number of snowflakes high</param>
        /// <returns></returns>
        protected static Vector2 BoardToScreen(int x, int y)
        {
            return new Vector2(LeftEdge + (float)x * Snowflake.Width,
                               TopEdge + (float)y * Snowflake.Height);
        }


        private void FindCursorPosition()
        {
            //If the cursor is on a snowflake then leave it there
            if (Snowflakes[cursorX, cursorY] != null) return;

            //We search the squares in this order....
            //   X  3  8  15
            //   1  2  7  14
            //   4  5  6  13
            //   9  10 11 12

            //Otherwise move to the closes snowflake down and to the right.
            for (int distance = 1;
                 distance < Math.Max(Width - cursorX, Height - cursorY) - 1;
                 distance++)
            {
                //Search below and across
                if (cursorY + distance < Height)
                {
                    for (int x = cursorX;
                         x < Math.Min(cursorX + distance + 1, Width); x++)
                    {
                        if (Snowflakes[x, cursorY + distance] != null)
                        {
                            cursorX = x;
                            cursorY = cursorY + distance;
                            return;
                        }
                    }
                }

                //Search to the right
                if (cursorX + distance < Width)
                {
                    for (int y = Math.Min(cursorY + distance, Height) - 1; y >= cursorY;
                         y--)
                    {
                        if (Snowflakes[cursorX + distance, y] != null)
                        {
                            cursorX = cursorX + distance;
                            cursorY = y;
                            return;
                        }
                    }
                }
            }

            //If we exit the loop then there is no place to go so they must have cleared
            //the board in which case its game over anyway
            // ?? If board is cleared, then need to generate a new board.
            return;
        }

        private void BreakSnowflakes(GameTime time)
        {

#if MOUSE
            bool leftButtonClicked = false;
            MouseState mouse = Mouse.GetState();
            if (mouse.LeftButton == ButtonState.Pressed)
            {
                leftButtonDown = true;
            }
            bool rightButtonClicked = false;
            if (mouse.RightButton == ButtonState.Pressed)
            {
                rightButtonDown = true;
            }

            if (leftButtonDown && mouse.LeftButton == ButtonState.Released)
            {
                leftButtonDown = false;
                leftButtonClicked = true;
            }

            if (rightButtonDown && mouse.RightButton == ButtonState.Released)
            {
                rightButtonDown = false;
                rightButtonClicked = true;
            }


#endif
            //break if possible
            bool burstPressed = InputHelper.GamePads[PlayerIndex.One].APressed
#if MOUSE
 || (Game.IsMouseVisible && leftButtonClicked)
#endif
;
            bool convertPressed = InputHelper.GamePads[PlayerIndex.One].XPressed
#if MOUSE
 || (Game.IsMouseVisible && rightButtonClicked)
#endif
;

            Snowflake currSnowflake = Snowflakes[cursorX, cursorY];

            if (burstPressed && currSnowflake != null)
            {   // We have a button pressed on an existing snowflake
                // Check to see if this matches our set snowflake
                if (isCorrectMatch(currSnowflake, SnowflakeToMatch))
                {   // Have a match of either shape or colour
                    // Check to see if this is an exact match
                    if (isExactMatch(currSnowflake, SnowflakeToMatch))
                    {
                        soundClearBonus.Play();
                        //Sound.Play(SoundEntry.ClearBonus);
                        // This will increase the score obtained
                        totalSelected += bonusExactMatch;
                    }
                    else
                    {
                        soundClear.Play();
                        //Sound.Play(SoundEntry.ClearSnowflakes);
                    }
                    Snowflakes[cursorX, cursorY].Selected = true;
                    SnowflakeToMatch.Selected = true;
                    totalSelected++;

                    // Increase score
                    SnowflakesGame.Score += Score(totalSelected);

                    // Break the snowflakes
                    AnimationState = Animation.Breaking;
                    foreach (Snowflake snowflake in Snowflakes)
                    {
                        if (snowflake != null && snowflake.Selected)
                        {
                            snowflake.Break(time);
                        }
                    }
                    // Break our match snowflake
                    SnowflakeToMatch.Break(time);

                    //Setup the score fading
                    scoreFadeStart = time;
                }
                else
                { // illegal move, incorrect match.  Clear selections
                    soundClearIllegal.Play();
                    //    Sound.Play(SoundEntry.ClearIllegal);
                    ResetSelectedSnowflakes();
                    // Reduce number of lives
                    SnowflakesGame.Lives--;
                    if (SnowflakesGame.IsShapeMatch)
                    {
                        if (!hadFirstWrongShape)
                        {
                            hadFirstWrongShape = true;
                            InfoScreenState = InfoScreen.WrongMatch;
                            // Pause the game
                            timeLeftAtPause = timeAllowedToMatch -
                                            (time.TotalGameTime.TotalSeconds - matchStartTime);
                        }
                    }
                    else
                    {
                        if (!hadFirstWrongColor)
                        {
                            hadFirstWrongColor = true;
                            InfoScreenState = InfoScreen.WrongMatch;
                            // Pause the game
                            timeLeftAtPause = timeAllowedToMatch -
                                            (time.TotalGameTime.TotalSeconds - matchStartTime);
                        }
                    }
                }
            }

            if (convertPressed)
            {   // Want to get a new match tile
                // Check to see if the match tile could have worked
                //  with ANY of the current tiles
                if (aMatchExists())
                {   // Shouldn't be getting a new tile
                    soundClearIllegal.Play();
                    //Sound.Play(SoundEntry.ClearIllegal);
                    // Reduce number of lives
                    SnowflakesGame.Lives--;
                    if (!hadFirstWrongConvert)
                    {
                        hadFirstWrongConvert = true;
                        InfoScreenState = InfoScreen.WrongMatch;
                        // Pause the game
                        timeLeftAtPause = timeAllowedToMatch -
                                        (time.TotalGameTime.TotalSeconds - matchStartTime);
                    }
                }
                // Get a new tile, so break the existing one
                AnimationState = Animation.Breaking;
                SnowflakeToMatch.Break(time);
            }

        }

        private Snowflake findFirstSelected()
        {
            foreach (Snowflake snowflake in Snowflakes)
            {
                if (snowflake != null && snowflake.Selected)
                {
                    return snowflake;
                }
            }
            // Couldn't find a selected snowflake
            return null;
        }

        private Snowflake findFirstSelected(Snowflake notThisSnowflake)
        {   // Finds the first selected snowflake, that isn't this snowflake
            foreach (Snowflake snowflake in Snowflakes)
            {
                if (snowflake != null && snowflake.Selected)
                {
                    if (notThisSnowflake == null || (notThisSnowflake != null
                            && !notThisSnowflake.Position.Equals(snowflake.Position)))
                    {
                        return snowflake;
                    }
                }
            }
            // Couldn't find a selected snowflake
            return null;
        }

        private Snowflake findRandomNotSelected(Snowflake notThisSnowflake)
        {   // Finds a random snowflake that is NOT selected AND that isn't this snowflake
            Snowflake newSnowflake = null;
            Random random = new Random();

            while (newSnowflake == null)
            {
                // Choose a random X & Y
                int randomX = random.Next(Width);
                int randomY = random.Next(Height);
                newSnowflake = Snowflakes[randomX, randomY];
                // Check to see if this is a valid snowflake
                if (newSnowflake != null && !newSnowflake.Selected)
                {
                    if (notThisSnowflake == null || (notThisSnowflake != null
                            && !notThisSnowflake.Position.Equals(newSnowflake.Position)))
                    {
                        return newSnowflake;
                    }
                    else
                    {   // Haven't found a match.  Need to keep looking
                        newSnowflake = null;
                    }
                }
            }
            // Couldn't find a selected snowflake
            return null;
        }

        private bool isCorrectMatch(Snowflake newSnowflake)
        {
            // Checks to see if the current snowflake and the selected snowflake match
            if (SnowflakesGame.IsShapeMatch)
            {
                return isCorrectShapeMatch(newSnowflake);
            }
            return isCorrectColorMatch(newSnowflake);
        }

        private bool isCorrectMatch(Snowflake snowflake1, Snowflake snowflake2)
        {
            // Checks to see if two snowflakes  match
            if (snowflake1 != null && snowflake2 != null
                    && !snowflake1.Position.Equals(snowflake2.Position))
            {
                if (SnowflakesGame.IsShapeMatch)
                {
                    return isCorrectShapeMatch(snowflake1, snowflake2);
                }
                return isCorrectColorMatch(snowflake1, snowflake2);
            }
            return false;
        }

        private bool aMatchExists()
        {
            foreach (Snowflake snowflake in Snowflakes)
            {
                if (isCorrectMatch(snowflake, SnowflakeToMatch))
                {   // Found a possible match
                    return true;
                }
            }
            return false;
        }


        private bool isExactMatch(Snowflake newSnowflake)
        {// Checks to see if both shape and color match
            // Finds the second selected snowflake first
            Snowflake selectedSnowflake = findFirstSelected();
            if (selectedSnowflake != null && newSnowflake != null
                    && !selectedSnowflake.Position.Equals(newSnowflake.Position))
            {
                return isExactMatch(newSnowflake, selectedSnowflake);
            }
            // Couldn't find a selected snowflake
            return false;
        }

        private bool isExactMatch(Snowflake snowflake1, Snowflake snowflake2)
        {// Compares two snowflakes based on shape AND color
            if (snowflake1 != null && snowflake2 != null
                    && !snowflake1.Position.Equals(snowflake2.Position))
            {
                return isCorrectShapeMatch(snowflake1, snowflake2)
                        && isCorrectColorMatch(snowflake1, snowflake2);
            }
            return false;
        }

        private bool isCorrectShapeMatch(Snowflake newSnowflake)
        {   // Checks to see if snowflakes match based on shape
            Snowflake selectedSnowflake = findFirstSelected();
            if (selectedSnowflake != null && newSnowflake != null
                    && !selectedSnowflake.Position.Equals(newSnowflake.Position))
            {
                return isCorrectShapeMatch(newSnowflake, selectedSnowflake);
            }
            // If got to here, haven't found a match
            return false;
        }

        private bool isCorrectShapeMatch(Snowflake snowflake1, Snowflake snowflake2)
        {   // Compares two snowflakes based on shape
            if (snowflake1 != null && snowflake2 != null
                    && !snowflake1.Position.Equals(snowflake2.Position))
            {
                if (snowflake1.ShapeNum == snowflake2.ShapeNum)
                {
                    return true;
                }
            }

            return false;
        }

        private bool isCorrectColorMatch(Snowflake newSnowflake)
        {   // Checks to see if snowflakes match based on color
            // Finds the selected snowflake first
            Snowflake selectedSnowflake = findFirstSelected();
            if (selectedSnowflake != null && newSnowflake != null
                    && !selectedSnowflake.Position.Equals(newSnowflake.Position))
            {
                return isCorrectColorMatch(newSnowflake, selectedSnowflake);
            }
            // Couldn't find a valid selected snowflake
            return false;
        }

        private bool isCorrectColorMatch(Snowflake snowflake1, Snowflake snowflake2)
        {   // Compares two snowflakes based on colour
            if (snowflake1 != null && snowflake2 != null
                   && !snowflake1.Position.Equals(snowflake2.Position))
            {
                if (snowflake1.ColorNum == snowflake2.ColorNum)
                {
                    return true;
                }
            }
            // If got to here, haven't found a match
            return false;
        }

        private void moveCursor()
        {
            //Move the cursor
            bool cursorMovedWithMouse = false;

#if MOUSE
            //If user moves the mouse then show it
            MouseState mouse = Mouse.GetState();
            if (mouse.X != lastMouseX && mouse.Y != lastMouseY)
            {
                //TODO: Doesn't seem to actually show the mouse until you move outside 
                //the window and back in.
                Game.IsMouseVisible = true;
            }

            //if the mouse is visible then track the cursor with the mouse
            if (Game.IsMouseVisible)
            {
                //Find which snowflake is under the cursor
                cursorMovedWithMouse = moveCursorWithMouse(mouse.X, mouse.Y);
                lastMouseX = mouse.X;
                lastMouseY = mouse.Y;
            }
#endif

            //Move the cursor with keyboard/gamepad
            bool cursorMovedWithGamePad = false;

            if (InputHelper.GamePads[PlayerIndex.One].LeftPressed)
            {
                cursorMovedWithGamePad = FindNextSnowflakeX(-1);
            }
            if (InputHelper.GamePads[PlayerIndex.One].RightPressed)
            {
                cursorMovedWithGamePad = FindNextSnowflakeX(1);
            }
            if (InputHelper.GamePads[PlayerIndex.One].UpPressed)
            {
                cursorMovedWithGamePad = FindNextSnowflakeY(-1);
            }
            if (InputHelper.GamePads[PlayerIndex.One].DownPressed)
            {
                cursorMovedWithGamePad = FindNextSnowflakeY(1);
            }

            //if anything moved we play the move sound
            //if (cursorMovedWithGamePad || cursorMovedWithMouse)
            //{
            //    Sound.Play(SoundEntry.Navigate);
            //FindSelectedSnowflakes();
            //}

#if MOUSE
            //If the user goes back to moving with the keyboard/gamepad 
            //then hide the mouse
            if (cursorMovedWithGamePad)
            {
                Game.IsMouseVisible = false;
            }
#endif

        }

#if MOUSE
        private bool moveCursorWithMouse(int x, int y)
        {
            bool clickedOnSnowflake = false;

            double scale = (float)GraphicsDevice.Viewport.Height / 400.0;
            double offset = (int)((GraphicsDevice.Viewport.Width - 700 * scale) / 2);

            int boardX = (int)((((x - offset) / scale) - LeftEdge) / Snowflake.Width);
            int boardY = (int)(((y / scale) - TopEdge) / Snowflake.Height);
            if (boardX >= 0 && boardX < Width && boardY >= 0 && boardY < Height
                //&& Snowflakes[boardX, boardY] != null
                )
            {
                //Don't actually move and play the sound if the mouse is moving within 
                //the snowflake
                if (cursorX != boardX || cursorY != boardY)
                {
                    cursorX = boardX;
                    cursorY = boardY;
                    clickedOnSnowflake = true;
                }
            }

            return clickedOnSnowflake;

        }
#endif

        private bool FindNextSnowflakeX(int direction)
        {
            //Wrap at the edge
            if (cursorX == 0 && direction == -1)
            {
                cursorX = GameBoard.Width;
            }
            else if (cursorX == (GameBoard.Width - 1) && direction == 1)
            {
                cursorX = -1;
            }

            //Move
            cursorX += direction;

            return true;
        }

        private bool FindNextSnowflakeY(int direction)
        {
            //Wrap at the edge
            if (cursorY == 0 && direction == -1)
            {
                cursorY = GameBoard.Height;
            }
            else if (cursorY == (GameBoard.Height - 1) && direction == 1)
            {
                cursorY = -1;
            }

            //Move
            cursorY += direction;

            return true;
        }



        private void ResetSelectedSnowflakes()
        {
            //Update the selected portions of the board
            foreach (Snowflake snowflake in Snowflakes)
            {
                if (snowflake != null)
                {
                    snowflake.Selected = false;
                }
            }
            SnowflakeToMatch.Selected = false;

            totalSelected = 0;
            //SelectSnowflake(cursorX, cursorY);
        }

        private void ClearBrokenSnowflakes()
        {
            // Clears snowflakes that were selected and have finished breaking
            for (int x = 0; x < Width; x++)
            {
                for (int y = Height - 1; y >= 0; y--)
                {
                    if (Snowflakes[x, y] != null && Snowflakes[x, y].Selected)
                    {
                        Snowflakes[x, y] = null;
                    }
                }
            }
        }

        private static int Score(int totalSelected)
        {
            return totalSelected * (scoreAmount);
        }

        private int estimateScore(int totalSelected, int cursorX, int cursorY)
        {   // Estimates what the score would be if this snowflake was selected
            // First check that the snowflake selected is not where the cursor is now.
            if (totalSelected == 0)
            {
                return 0;
            }
            if (totalSelected > 1)
            {   // Already chosen our two snowflakes
                // Just output current score
                Score(totalSelected);
            }
            Snowflake currentSnowflake = Snowflakes[cursorX, cursorY];
            Snowflake selectedSnowflake = findFirstSelected(currentSnowflake);
            if (selectedSnowflake != null && currentSnowflake != null
                    && !selectedSnowflake.Position.Equals(currentSnowflake.Position))
            {
                if (isCorrectMatch(selectedSnowflake, currentSnowflake))
                {   // Have a match
                    if (isExactMatch(selectedSnowflake, currentSnowflake))
                    {
                        return Score(totalSelected + bonusExactMatch);
                    }
                    return Score(totalSelected);
                }
            }
            // Haven't found a good match yet.
            return 0;
        }

        /// <summary>
        /// Draws the game board and all child objects
        /// </summary>
        /// <param name="spriteBatch">a sprite batch to use to draw any sprites</param>
        public void Draw(RelativeSpriteBatch spriteBatch)
        {
            if (spriteBatch == null)
                return;

            foreach (Snowflake snowflake in Snowflakes)
            {
                if (snowflake != null)
                {
                    snowflake.Draw(spriteBatch);
                }
            }

            // Draw the match tile only if no info screen up
            if (InfoScreenState == InfoScreen.None)
            {
                SnowflakeToMatch.Draw(spriteBatch);
            }

            if (!GameOver)
            {
                if (InfoScreenState == InfoScreen.None)
                {
                    // Draw the time left to match
                    for (int i = 0; i < 2 * timeLeftInteger; i++)
                    {
                        spriteBatch.Draw(timeSegmentImgSmall, new Vector2(50 + i * 10, 367), Color.White);
                        //Font.Draw(spriteBatch, FontStyle.Small, 250, 30,
                        //          String.Format("{0:###,##0}", timeLeftInteger));
                    }
                    //Draw the cursor
                    spriteBatch.Draw(snowflakeCursorTexture, BoardToScreen(cursorX, cursorY),
                                      Color.White);
                }

                if (totalSelected > 0)
                {
                    Vector2 position = BoardToScreen(cursorX, cursorY) + scoreOffset;
                    Color color = Color.White;

                    if (AnimationState == Animation.Breaking && totalSelected > 0)
                    {
                        // Slide the score up and fade it
                        position -= new Vector2(0, scoreFadeFactor * scoreFadeDistance);
                        color = new Color(255, 255, 255,
                                          (byte)(255 * (1f - scoreFadeFactor)));
                        Font.Draw(spriteBatch, FontStyle.LargeWhite, position,
                                   Score(totalSelected), color);
                    }
                }

            }
        }


        /// <summary>
        /// Returns the state of the board as a string - useful for debugging
        /// </summary>
        /// <returns>String representation of the board</returns>
        public override string ToString()
        {
            StringBuilder board = new StringBuilder();
            for (int y = 0; y < Height; y++)
            {
                for (int x = 0; x < Width; x++)
                {
                    if (Snowflakes[x, y] == null)
                        board.Append("--null-- ");
                    else
                        board.Append(Snowflakes[x, y].Color.ToString() + " ");
                }
                board.AppendLine();
            }
            board.AppendLine();
            return board.ToString();
        }

        /// <summary>
        /// Check to see if the game is over or not. This is a function rather than a 
        /// property to indicate that it does significant
        /// work rather than being a cheap property get
        /// </summary>
        /// <returns>true is the game is over</returns>
        private bool IsGameOver()
        {
            if (SnowflakesGame.Lives > 0)
            {
                // Still have some lives
                return false;
            }

            //Game is over.
            SaveHighScores();

            return true;
        }

        /// <summary>
        /// Check to see if the screen is clear yet.
        /// If it is.  We need to generate a new screen and do some action
        /// </summary>
        /// <returns>true is the game screen is clear of snowflakes</returns>
        private void checkScreenClear(GameTime gameTime)
        {
            foreach (Snowflake snowflake in Snowflakes)
            {
                if (snowflake != null)
                {   // Found a snowflake, so game screen is not clear yet
                    return;
                }
            }

            // Didn't find any snowflakes
            // Create a new game screen
            soundNewScreen.Play();
            NewScreen(gameTime);
            // Give the player a bonus
            SnowflakesGame.Score += bonusClearScreen;
            // Give the player a new life
            SnowflakesGame.Lives++;
            // Reduce the time between squares OR reduce match type switching
            // Reduce time more frequently
            if (changeTimeAmount > 0)
            {   // Change the amount of time allowed to find a match
                if (timeAllowedToMatch > timeAllowedToMatchDecrease)
                {
                    timeAllowedToMatch -= timeAllowedToMatchDecrease;
                }
                else
                {   // Reached the minimum amount
                    timeAllowedToMatch = minTimeAllowedToMatch;
                }
                changeTimeAmount--;
            }
            else
            {   // Change how often switch between shapes and colours
                if (SnowflakesGame.SwitchFrequencyMatchType > 1)
                {
                    SnowflakesGame.SwitchFrequencyMatchType--;
                    if (SnowflakesGame.NumMatchesUntilSwitchType > SnowflakesGame.SwitchFrequencyMatchType)
                    {   // Have decresed the amount, but not the actual count left yet
                        SnowflakesGame.NumMatchesUntilSwitchType
                            = SnowflakesGame.SwitchFrequencyMatchType;
                    }
                }
                else
                {
                    // Have reached the time where will change every step
                    if (!hadFirstSwitchEveryTime)
                    {
                        // This is the first time we got here.
                        // Give a message so know that this will keep happening all the time
                        hadFirstSwitchEveryTime = true;
                        InfoScreenState = InfoScreen.SwitchMatch;
                    }
                }
                changeTimeAmount = changeTimeOccurence;
            }
            // Are we doing a new screen while still in the middle of 
            //  shapes or colours?
            // Need to give reminder that still matching shapes/colours
            if (!hadFirstSwitchInMiddleOfScreen &&
                    (SnowflakesGame.NumMatchesUntilSwitchType < SnowflakesGame.SwitchFrequencyMatchType))
            {
                hadFirstSwitchInMiddleOfScreen = true;
                InfoScreenState = InfoScreen.SwitchMatch;
            }
            return;
        }

        /// <summary>
        /// Save the high scores to the drive.
        /// </summary>
        private static void SaveHighScores()
        {
            //Insert the score into the high score table
            for (int i = 0; i < 5; i++)
            {
                if (SnowflakesGame.Score > SnowflakesGame.HighScores[i])
                {
                    //Insert new score
                    SnowflakesGame.HighScores.Insert(i, SnowflakesGame.Score);
                    //And remove the lowest from the bottom
                    SnowflakesGame.HighScores.RemoveAt(5);
                    break;
                }
            }

            if ((SnowflakesGame.StorageDevice != null) &&
                SnowflakesGame.StorageDevice.IsConnected)
            {
                SaveHighScoresCallback(null);
            }
            else
            {
                Guide.BeginShowStorageDeviceSelector(
                    new AsyncCallback(SaveHighScoresCallback), null);
            }
        }

        /// <summary>
        /// Callback method for saving the high scores to the drive.
        /// </summary>
        private static void SaveHighScoresCallback(IAsyncResult result)
        {
            if ((result != null) && result.IsCompleted)
            {
                SnowflakesGame.StorageDevice = Guide.EndShowStorageDeviceSelector(result);
            }
            if ((SnowflakesGame.StorageDevice != null) &&
                SnowflakesGame.StorageDevice.IsConnected)
            {
                using (StorageContainer storageContainer =
                    SnowflakesGame.StorageDevice.OpenContainer("Snowflakes"))
                {
                    string highscoresPath = Path.Combine(storageContainer.Path,
                                                         "highscores.xml");
                    using (FileStream file = File.Create(highscoresPath))
                    {
                        XmlSerializer serializer = new XmlSerializer(typeof(List<int>));
                        serializer.Serialize(file, SnowflakesGame.HighScores);
                    }
                }
            }
        }

        /// <summary>
        /// Initializes the board
        /// </summary>
        public void NewGame(GameTime gameTime)
        {
            gameOver = false;

            SnowflakeToMatch = new Snowflake(random.Next());
            SnowflakeToMatch.Position = SnowflakeToMatchPosition;

            //Fill board
            for (int x = 0; x < Width; x++)
            {
                for (int y = 0; y < Height; y++)
                {
                    Snowflakes[x, y] = new Snowflake(random.Next());
                    Snowflakes[x, y].Position = BoardToScreen(x, y);
                }
            }

            cursorX = 0;
            cursorY = 0;

            // Start the timer
            timeAllowedToMatch = maxTimeAllowedToMatch;
            matchStartTime = gameTime.TotalGameTime.TotalSeconds;
            timeLeftInteger = Convert.ToInt32(timeAllowedToMatch);

            //The cursor might be on a matching color set
            //FindSelectedSnowflakes();
        }

        /// <summary>
        /// Generates a new screen with new snowflakes
        /// </summary>
        public void NewScreen(GameTime gameTime)
        {
            gameOver = false;

            SnowflakeToMatch = new Snowflake(random.Next());
            SnowflakeToMatch.Position = SnowflakeToMatchPosition;

            //Fill board
            for (int x = 0; x < Width; x++)
            {
                for (int y = 0; y < Height; y++)
                {
                    Snowflakes[x, y] = new Snowflake(random.Next());
                    Snowflakes[x, y].Position = BoardToScreen(x, y);
                }
            }

            // Start the timer
            matchStartTime = gameTime.TotalGameTime.TotalSeconds;
            timeLeftInteger = Convert.ToInt32(timeAllowedToMatch);
        }
    }
}
