#region File Description
//-----------------------------------------------------------------------------
// GameScreen.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

// #define TutorialVersion //Uncomment this line to play the tutorial version

#region Using Statements
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace Snowflakes
{
    /// <summary>
    /// GameScreen is the screen that is loaded to play Snowflakes
    /// </summary>
    class GameScreen : Screen
    {
        private GameBoard board;
        private Texture2D instructions;
        private Texture2D instructionsShapes;
        private Texture2D gameOver;
        //private Texture2D matchShapes;
        //private Texture2D matchColors;
        //private Texture2D matchShapesNegative;
        //private Texture2D matchColorsNegative;
        private Texture2D switchEveryTime;
        private Texture2D switchToShapes;
        private Texture2D switchToColors;
        private Texture2D stillOnShapes;
        private Texture2D stillOnColors;
        private Texture2D pauseShapes;
        private Texture2D pauseColors;
        private Texture2D wrongShapes;
        private Texture2D wrongColors;
        private Texture2D wrongTimeUp;

        public GameScreen(Game game, string backgroundImage)
            : base(game, backgroundImage)
        {

            //Remove the define at the top of the file to play the tutorial version
            bool tutorialVersion = false;
#if TutorialVersion
            tutorialVersion = true;
#endif
            if (tutorialVersion)
            {
                board = new TutorialGameBoard(this.Game);
            }
            else
            {
                board = new GameBoard(this.Game);
            }
        }

        public override void Initialize()
        {
            base.Initialize();
            board.Initialize();
        }

        public void ShowInstructions()
        {
            board.setShowInstructions();
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);
            board.Update(gameTime);

            if (board.GameOver && InputHelper.GamePads[PlayerIndex.One].APressed)
            {
                SnowflakesGame.NextGameState = GameState.Started;
            }
        }

        protected override void LoadContent()
        {
            instructions = SnowflakesGame.Content.Load<Texture2D>("Textures/initialInstructions");
            instructionsShapes = SnowflakesGame.Content.Load<Texture2D>("Textures/initialInstructionsShapes");
            gameOver = SnowflakesGame.Content.Load<Texture2D>("Textures/game_over_frame");
            //matchShapes = SnowflakesGame.Content.Load<Texture2D>("Textures/match_shapes");
            //matchColors = SnowflakesGame.Content.Load<Texture2D>("Textures/match_colours");
            //matchShapesNegative = SnowflakesGame.Content.Load<Texture2D>("Textures/match_shapes_negative");
            //matchColorsNegative = SnowflakesGame.Content.Load<Texture2D>("Textures/match_colours_negative");
            switchEveryTime = SnowflakesGame.Content.Load<Texture2D>("Textures/switchEveryTime");
            switchToColors = SnowflakesGame.Content.Load<Texture2D>("Textures/switchToColors");
            switchToShapes = SnowflakesGame.Content.Load<Texture2D>("Textures/switchToShapes");
            stillOnColors = SnowflakesGame.Content.Load<Texture2D>("Textures/stillOnColors");
            stillOnShapes = SnowflakesGame.Content.Load<Texture2D>("Textures/stillOnShapes");
            pauseColors = SnowflakesGame.Content.Load<Texture2D>("Textures/pauseColors");
            pauseShapes = SnowflakesGame.Content.Load<Texture2D>("Textures/pauseShapes");
            wrongShapes = SnowflakesGame.Content.Load<Texture2D>("Textures/wrongShapes");
            wrongColors = SnowflakesGame.Content.Load<Texture2D>("Textures/wrongColors");
            wrongTimeUp = SnowflakesGame.Content.Load<Texture2D>("Textures/timeUpWrong");

            base.LoadContent();
        }


        public override void Draw(GameTime gameTime)
        {
            base.Draw(gameTime);

            SpriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred,
                              SaveStateMode.None);

            board.Draw(SpriteBatch);

            //Draw Score
            Font.Draw(SpriteBatch, FontStyle.Small, 550, 167,
                      String.Format("{0:###,##0}", SnowflakesGame.Score));

            // Draw new lives left
            Font.Draw(SpriteBatch, FontStyle.Large, 90, 152,
                      String.Format("{0:###,##0}", SnowflakesGame.Lives));

            // Draw match type
            //if (SnowflakesGame.IsShapeMatch)
            //{
            //    SpriteBatch.Draw(matchShapes, new Vector2(275, 130), Color.White);
            //    SpriteBatch.Draw(matchColorsNegative, new Vector2(350, 130), Color.White);
            //}
            //else
            //{
            //    SpriteBatch.Draw(matchShapesNegative, new Vector2(275, 130), Color.White);
            //    SpriteBatch.Draw(matchColors, new Vector2(350, 130), Color.White);
            //}
            // Draw number left until switch match type
            //for (int i=0; i<SnowflakesGame.NumMatchesUntilSwitchType; i++)
            //{
            //    SpriteBatch.Draw(board.timeSegmentImg, new Vector2(275+i*19, 210), Color.White);
            //}
            //Font.Draw(SpriteBatch, FontStyle.Small, 20, 36,
            //          String.Format("{0:###,##0}", ));

            if (board.GameOver)
            {
                SpriteBatch.Draw(gameOver, new Vector2(150, 100), Color.White);
            }
            else if (board.needToShowInfo())
            {
                switch (board.InfoScreenState)
                {
                    case InfoScreen.Instructions:
                        {
                            if (SnowflakesGame.IsShapeMatch)
                            {
                                SpriteBatch.Draw(instructionsShapes, new Vector2(50, 25), Color.White);
                            }
                            else
                            {
                                SpriteBatch.Draw(instructions, new Vector2(50, 25), Color.White);
                            }
                            break;
                        }
                    case InfoScreen.WrongMatch:
                        {
                            if (SnowflakesGame.IsShapeMatch)
                            {
                                SpriteBatch.Draw(wrongShapes, new Vector2(250, 130), Color.White);
                            }
                            else
                            {
                                SpriteBatch.Draw(wrongColors, new Vector2(250, 130), Color.White);
                            }
                            break;
                        }
                    case InfoScreen.SwitchMatch:
                        {
                            if (SnowflakesGame.NumMatchesUntilSwitchType < SnowflakesGame.SwitchFrequencyMatchType)
                            {
                                // We changed screens in the middle of matching something
                                if (SnowflakesGame.IsShapeMatch)
                                {
                                    SpriteBatch.Draw(stillOnShapes, new Vector2(250, 130), Color.White);
                                }
                                else
                                {
                                    SpriteBatch.Draw(stillOnColors, new Vector2(250, 130), Color.White);
                                }
                                // Just need to put a reminder up.
                            }
                            else if (SnowflakesGame.SwitchFrequencyMatchType == 1)
                            {
                                // Have reached the stage where switch match every time
                                // Give a warning first time, then don't show warnings
                                //  anymore.
                                // Won't show warnings next time, because it won't be
                                //  triggered to show infoscreen from gameboard
                                SpriteBatch.Draw(switchEveryTime, new Vector2(200, 100), Color.White);
                            }
                            else
                            {   // We are changing the match type normally, set reminder.
                                if (SnowflakesGame.IsShapeMatch)
                                {
                                    SpriteBatch.Draw(switchToShapes, new Vector2(250, 130), Color.White);
                                }
                                else
                                {
                                    SpriteBatch.Draw(switchToColors, new Vector2(250, 130), Color.White);
                                }
                            }
                            break;
                        }
                    case InfoScreen.TimeUp:
                        {
                            SpriteBatch.Draw(wrongTimeUp, new Vector2(250, 130), Color.White);
                            break;
                        }
                    case InfoScreen.PauseInfo:
                        {
                            if (SnowflakesGame.IsShapeMatch)
                            {
                                SpriteBatch.Draw(pauseShapes, new Vector2(50, 25), Color.White);
                            }
                            else
                            {
                                SpriteBatch.Draw(pauseColors, new Vector2(50, 25), Color.White);
                            }
                            break;
                        }
                }
            }

            SpriteBatch.End();

        }

        public void NewGame(GameTime gameTime)
        {
            board.NewGame(gameTime);
        }

        /// <summary>
        /// Checks to see if the game is paused
        /// </summary>
        /// <returns> True if the game is paused</returns>
        public bool isPaused()
        {
            if (board.InfoScreenState == InfoScreen.PauseInfo)
            {
                return true;
            }
            return false;
        }
    }
}
