#region File Description
//-----------------------------------------------------------------------------
// Snowflake.cs
//
// Microsoft XNA Community Game Platform
// Copyright (C) Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#endregion

#region Using Statements
using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace Snowflakes
{
    /// <summary>
    /// Snowflake represents a single snowflake
    /// </summary>
    public class Snowflake
    {
        /// <summary>
        /// Width of a snowflake in HD resolution screen coordinates
        /// </summary>
        public const int Width = 75;

        /// <summary>
        /// Height of a snowflake in HD resolution screen coordinates
        /// </summary>
        public const int Height = 75;

        /// <summary>
        /// How long it takes a snowflake to break
        /// </summary>
        public const double BreakTime = 0.5;

        /// <summary>
        /// Animation fields
        /// </summary>
        private const int breakFrameCount = 12;
        private static float pulseFactor;
        private double animationStart;
        private int frame;
        private Animation animation = Animation.None;

        /// <summary>
        /// Initialize the random number generator
        /// </summary>
        private static Random random = new Random();

        /// <summary>
        /// Snowflake textures
        /// </summary>
        private static Texture2D breakTexture;
        private static Texture2D snowflake1Texture;
        private static Texture2D snowflake2Texture;
        private static Texture2D snowflake3Texture;
        private static Texture2D snowflake4Texture;
        private static Texture2D snowflake5Texture;
        private static Texture2D snowflake6Texture;
        private static Texture2D snowflake7Texture;
        private static Texture2D snowflake8Texture;
        private static Texture2D snowflake1MiniTexture;
        private static Texture2D snowflake3MiniTexture;
        private static Texture2D snowflake4MiniTexture;
        private static Texture2D snowflake7MiniTexture;
        private static Texture2D snowflake8MiniTexture;
        private Color color;
        private int colorNum;
        private String shape;
        private int shapeNum;

        /// <summary>
        /// Snowflake position
        /// </summary>
        private Vector2 position2D;




        /// <summary>
        /// Current state of the animation
        /// </summary>
        public Animation Animation
        {
            get
            {
                return animation;
            }
        }

        /// <summary>
        /// Color of the snowflake
        /// </summary>
        public Color Color
        {
            get
            {
                return color;
            }
        }
        public int ColorNum
        {
            get
            {
                return colorNum;
            }
        }

        /// <summary>
        /// Shape of the snowflake
        /// </summary>
        public String Shape
        {
            get
            {
                return shape;
            }
        }
        public int ShapeNum
        {
            get
            {
                return shapeNum;
            }
        }
        public Texture2D getSnowflakeTexture()
        {
            if (shapeNum == 0)
            {
                return snowflake1Texture;
            }
            else if (shapeNum == 1)
            {
                return snowflake2Texture;
            }
            else if (shapeNum == 2)
            {
                return snowflake3Texture;
            }
            else if (shapeNum == 3)
            {
                return snowflake4Texture;
            }
            else if (shapeNum == 4)
            {
                return snowflake5Texture;
            }
            else if (shapeNum == 5)
            {
                return snowflake6Texture;
            }
            else if (shapeNum == 6)
            {
                return snowflake7Texture;
            }
            else if (shapeNum < 0)
            {   // Have a mini shape
                if (shape.Equals("snowflake1Mini"))
                {
                    return snowflake1MiniTexture;
                }
                else if (shape.Equals("snowflake2Mini"))
                {
                    return snowflake3MiniTexture;
                }
                else if (shape.Equals("snowflake3Mini"))
                {
                    return snowflake4MiniTexture;
                }
                else if (shape.Equals("snowflake4Mini"))
                {
                    return snowflake7MiniTexture;
                }
                else
                {
                    return snowflake8MiniTexture;
                }
            }

            return snowflake8Texture;
        }

        /// <summary>
        /// Is this snowflake currently selected
        /// </summary>
        public bool Selected;

        /// <summary>
        /// The HD screen based coordinates of the snowflake
        /// </summary>
        public Vector2 Position
        {
            get
            {
                return position2D;
            }

            set
            {
                position2D = value;
            }
        }

        /// <summary>
        /// Constructs a new snowflake
        /// </summary>
        public Snowflake(int seed)
        {
            random = new Random(seed);

            //Set a random color
            colorNum = random.Next(SnowflakesGame.Settings.SnowflakeColors.GetLength(0));
            color = SnowflakesGame.Settings.SnowflakeColors[colorNum];

            //Set a random shape
            shapeNum = random.Next(SnowflakesGame.Settings.SnowflakeShapes.GetLength(0));
            shape = SnowflakesGame.Settings.SnowflakeShapes[shapeNum];

        }

        /// <summary>
        /// Constructs a new mini snowflake
        /// </summary>
        public Snowflake(string fixed_shape, int i)
        {
            //Set a random color but don't choose white
            colorNum = (i % 3) + 1;
            color = SnowflakesGame.Settings.SnowflakeColors[colorNum];

            //Set shape according to input value
            shapeNum = -1 * (i + 1);
            shape = "snowflake" + (i + 1) + "Mini";

            //Width = 50;
            //Height = 50;

        }

        /// <summary>
        /// Constructs a new snowflake in a specific position
        /// </summary>
        public Snowflake(Vector2 newPosition)
        {
            position2D = newPosition;

            //Set a random color
            colorNum = random.Next(SnowflakesGame.Settings.SnowflakeColors.GetLength(0));
            color = SnowflakesGame.Settings.SnowflakeColors[colorNum];

            //Set a random shape
            shapeNum = random.Next(SnowflakesGame.Settings.SnowflakeShapes.GetLength(0));
            shape = SnowflakesGame.Settings.SnowflakeShapes[shapeNum];

        }


        /// <summary>
        /// Initializes the snowflake
        /// </summary>
        /// <param name="game">The game object</param>
        public static void Initialize()
        {
        }

        /// <summary>
        /// Updates the animation for snowflakes
        /// </summary>
        /// <param name="time">Total game time</param>
        public void Update(GameTime time)
        {
            if (time != null)
            {
                if (Animation == Animation.Breaking)
                {
                    if ((time.TotalGameTime.TotalSeconds - animationStart) > BreakTime)
                    {
                        animation = Animation.Gone;
                        frame = breakFrameCount;
                    }
                    else
                    {
                        frame = (int)((time.TotalGameTime.TotalSeconds - animationStart)
                                     / BreakTime * breakFrameCount);
                    }
                }
            }
        }

        /// <summary>
        /// Updates the static parts of the class - only needs to be done once rather 
        /// than for 144 snowflakes each frame
        /// </summary>
        /// <param name="time">Total game time</param>
        public static void UpdateStatic(GameTime time)
        {
            if (time != null)
                pulseFactor = (float)Math.Sin(time.TotalGameTime.TotalSeconds * 6.0);
        }

        /// <summary>
        /// Draws the specified snowflake
        /// </summary>
        /// <param name="spriteBatch">The sprite batch to use</param>
        public void Draw(RelativeSpriteBatch spriteBatch)
        {
            Draw2DSnowflake(spriteBatch);
        }

        private void Draw2DSnowflake(RelativeSpriteBatch spriteBatch)
        {
            //Draw the 2d snowflake
            if (Animation == Animation.Breaking || Animation == Animation.Gone)
            {
                //Draw the correct frame.
                spriteBatch.Draw(breakTexture, Position, new Rectangle(frame * Width, 0,
                                 Width, Height), Color);
            }
            else
            {
                spriteBatch.Draw(getSnowflakeTexture(), Position, Color);
            }
        }

        /// <summary>
        /// Signal a snowflake to start breaking
        /// </summary>
        /// <param name="time">Start time for the animation</param>
        public void Break(GameTime time)
        {
            if (time != null)
            {
                animationStart = time.TotalGameTime.TotalSeconds;
                animation = Animation.Breaking;
            }
        }

        /// <summary>
        /// Converts a snowflake to a new shape or color, RANDOMLY
        /// </summary>
        public void convert()
        {
            // Remember old color
            // Make sure one of color and shape changes
            int oldShapeNum = shapeNum;
            int oldColorNum = colorNum;

            while (oldShapeNum == shapeNum && oldColorNum == colorNum)
            {
                //Set a random color
                colorNum = random.Next(SnowflakesGame.Settings.SnowflakeColors.GetLength(0));
                color = SnowflakesGame.Settings.SnowflakeColors[colorNum];

                //Set a random shape
                shapeNum = random.Next(SnowflakesGame.Settings.SnowflakeShapes.GetLength(0));
                shape = SnowflakesGame.Settings.SnowflakeShapes[shapeNum];
            }

            return;
        }

        /// <summary>
        /// Converts a snowflake to a new shape or color so it will match a tile
        ///   that already exists
        /// </summary>
        public void convert(bool isShapeMatch, Snowflake otherSnowflake)
        {
            if (otherSnowflake == null)
            {
                // No snowflake to match with
                convert();
            }
            else
            {
                // Find out what sort of match we want
                if (isShapeMatch)
                {   // Want to have the same shape as the other snowflake
                    //Set a random color, since we don't care about that
                    colorNum = random.Next(SnowflakesGame.Settings.SnowflakeColors.GetLength(0));
                    color = SnowflakesGame.Settings.SnowflakeColors[colorNum];

                    //Set shape to match otherSnowflake
                    shapeNum = otherSnowflake.ShapeNum;
                    shape = otherSnowflake.Shape;
                }
                else
                {   // Must want to match on color
                    //Set color to match the other snowflake
                    colorNum = otherSnowflake.ColorNum;
                    color = otherSnowflake.Color;

                    //Set a random shape, since we don't care about shape
                    shapeNum = random.Next(SnowflakesGame.Settings.SnowflakeShapes.GetLength(0));
                    shape = SnowflakesGame.Settings.SnowflakeShapes[shapeNum];
                }
            }
            return;
        }

        /// <summary>
        /// Load graphics content.
        /// </summary>
        public static void LoadContent()
        {
            breakTexture =
                SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake_burst");
            snowflake1Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake1");
            snowflake2Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake2");
            snowflake3Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake3");
            snowflake4Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake4");
            snowflake5Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake5");
            snowflake6Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake6");
            snowflake7Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake7");
            snowflake8Texture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake8");
            snowflake1MiniTexture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake1_mini");
            snowflake3MiniTexture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake3_mini");
            snowflake4MiniTexture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake4_mini");
            snowflake7MiniTexture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake7_mini");
            snowflake8MiniTexture = SnowflakesGame.Content.Load<Texture2D>("Textures/snowflake8_mini");
        }
    }
}
